( function() {

    'use stric';

    var jwt = require( 'jwt-simple' );
    var rp = require( 'request-promise' );

    const defaults = {
        header: 'Authorization',
        jwtPublicKey: undefined,
        validationEndpoint: undefined,
        errorStatus: 498
    };

    function sendInvalidToken( next, o ) {
        var err = new Error( 'Token inválido.' );
        err.status = o.errorStatus;
        err.handled = true;

        next( err );
    }

    function validateJWTToken( o ) {
        return ( req, res, next ) => {
            try {
                const authorization = req.get( o.header );
                const token = authorization.slice( 7 );
                const decodedToken = jwt.decode( token, o.jwtPublicKey );

                if ( decodedToken.error ) {
                    sendInvalidToken( next, o );
                } else {
                    req.decodedToken = decodedToken;
                    next();
                }
            } catch ( err ) {
                sendInvalidToken( next, o );
            }
        };
    }

    function authenticationWrapper( options ) {
        if ( !options || options === true ) {
            options = {};
        }
        if ( options.header === undefined ) {
            options.header = defaults.header;
        }
        if ( options.errorStatus === undefined ) {
            options.errorStatus = defaults.errorStatus;
        }

        let pipeline = [];

        if ( options.jwtPublicKey ) {
            pipeline.push( validateJWTToken( options ) );
        }

        return pipeline;
    }

    function validateAtEndpoint( o ) {
        return ( req, res, next ) => {
            var options = {
                method: 'GET',
                uri: o.validationEndpoint,
                headers: {
                    'User-Agent': 'prodest-authentication-middleware',
                    'Authorization': req.get( o.header )
                }
            };

            return rp( options )
            .then( resp => {
                req.userInfo = resp;
                next();
            } )
            .catch( () => {
                sendInvalidToken( next, o );
            } );
        };
    }

    module.exports = {
        middleware: authenticationWrapper,
        validateAtEndpoint: validateAtEndpoint
    };

}() );
